<?php
/*
 * @copyright Copyright (c) 2023 AltumCode (https://altumcode.com/)
 *
 * This software is exclusively sold through https://altumcode.com/ by the AltumCode author.
 * Downloading this product from any other sources and running it without a proper license is illegal,
 *  except the official ones linked from https://altumcode.com/.
 */

namespace Altum\Controllers;

use Altum\Alerts;

class Vcards extends Controller {

    public function index() {
        \Altum\Authentication::guard();

        /* Get available custom domains */
        $domains = (new \Altum\Models\Domain())->get_available_domains_by_user($this->user, false);

        /* Prepare the filtering system */
        $filters = (new \Altum\Filters(['is_enabled', 'project_id', 'domain_id', 'pixels_ids'], ['name', 'url'], ['last_datetime', 'datetime', 'name', 'url', 'pageviews'], [], ['pixels_ids' => 'json_contains']));
        $filters->set_default_order_by('vcard_id', $this->user->preferences->default_order_type ?? settings()->main->default_order_type);
        $filters->set_default_results_per_page($this->user->preferences->default_results_per_page ?? settings()->main->default_results_per_page);

        /* Prepare the paginator */
        $total_rows = database()->query("SELECT COUNT(*) AS `total` FROM `vcards` WHERE `user_id` = {$this->user->user_id} {$filters->get_sql_where()}")->fetch_object()->total ?? 0;
        $paginator = (new \Altum\Paginator($total_rows, $filters->get_results_per_page(), $_GET['page'] ?? 1, url('vcards?' . $filters->get_get() . '&page=%d')));

        /* Get the vcards */
        $vcards = [];
        $vcards_result = database()->query("
            SELECT
                *
            FROM
                `vcards`
            WHERE
                `user_id` = {$this->user->user_id}
                {$filters->get_sql_where()}
                {$filters->get_sql_order_by()}

            {$paginator->get_sql_limit()}
        ");
        while($row = $vcards_result->fetch_object()) {
            $row->full_url = (new \Altum\Models\Vcard())->get_vcard_full_url($row, $this->user, $domains);
            $row->settings = json_decode($row->settings ?? '');
            $vcards[] = $row;
        }

        /* Export handler */
        process_export_csv($vcards, 'include', ['vcard_id', 'user_id', 'domain_id', 'project_id', 'pixels_ids', 'url', 'name', 'description', 'pageviews', 'is_se_visible', 'is_removed_branding', 'is_enabled', 'datetime'], sprintf(l('vcards.title')));
        process_export_json($vcards, 'include', ['vcard_id', 'user_id', 'domain_id', 'project_id', 'pixels_ids', 'url', 'name', 'description', 'pageviews', 'is_se_visible', 'is_removed_branding', 'is_enabled', 'datetime'], sprintf(l('vcards.title')));

        /* Prepare the pagination view */
        $pagination = (new \Altum\View('partials/pagination', (array) $this))->run(['paginator' => $paginator]);

        $projects = (new \Altum\Models\Projects())->get_projects_by_user_id($this->user->user_id);

        /* Get statistics */
        if(count($vcards) && !$filters->has_applied_filters) {
            $start_date_query = (new \DateTime())->modify('-30 day')->format('Y-m-d H:i:s');
            $end_date_query = (new \DateTime())->modify('+1 day')->format('Y-m-d H:i:s');

            $statistics_result_query = "
                SELECT
                    COUNT(`id`) AS `pageviews`,
                    SUM(`is_unique`) AS `visitors`,
                    DATE_FORMAT(`datetime`, '%Y-%m-%d') AS `formatted_date`
                FROM
                    `statistics`
                WHERE   
                    `user_id` = {$this->user->user_id} 
                    AND (`datetime` BETWEEN '{$start_date_query}' AND '{$end_date_query}')
                GROUP BY
                    `formatted_date`
                ORDER BY
                    `formatted_date`
            ";

            $vcards_chart = \Altum\Cache::cache_function_result('statistics?user_id=' . $this->user->user_id, null, function() use ($statistics_result_query) {
                $vcards_chart = [];

                $statistics_result = database()->query($statistics_result_query);

                /* Generate the raw chart data and save logs for later usage */
                while($row = $statistics_result->fetch_object()) {
                    $label = \Altum\Date::get($row->formatted_date, 5);

                    $vcards_chart[$label] = [
                        'pageviews' => $row->pageviews,
                        'visitors' => $row->visitors
                    ];
                }

                return $vcards_chart;
            }, 60 * 60 * settings()->vcards->vcards_chart_cache ?? 12);

            $vcards_chart = get_chart_data($vcards_chart);
        }

        /* Prepare the View */
        $data = [
            'vcards_chart' => $vcards_chart ?? null,
            'domains' => $domains,
            'projects' => $projects,
            'vcards' => $vcards,
            'total_vcards' => $total_rows,
            'pagination' => $pagination,
            'filters' => $filters,
        ];

        $view = new \Altum\View('vcards/index', (array) $this);

        $this->add_view_content('content', $view->run($data));
    }

    public function duplicate() {
        \Altum\Authentication::guard();

        /* Team checks */
        if(\Altum\Teams::is_delegated() && !\Altum\Teams::has_access('create.vcards')) {
            Alerts::add_info(l('global.info_message.team_no_access'));
            redirect('vcards');
        }

        if(empty($_POST)) {
            redirect('vcards');
        }

        /* Make sure that the user didn't exceed the limit */
        $total_rows = db()->where('user_id', $this->user->user_id)->getValue('vcards', 'COUNT(*)') ?? 0;
        if($this->user->plan_settings->vcards_limit != -1 && $total_rows >= $this->user->plan_settings->vcards_limit) {
            Alerts::add_info(l('global.info_message.plan_feature_limit'));
            redirect('vcards');
        }

        $vcard_id = (int) $_POST['vcard_id'];

        //ALTUMCODE:DEMO if(DEMO) if($this->user->user_id == 1) Alerts::add_error('Please create an account on the demo to test out this function.');

        if(!\Altum\Csrf::check()) {
            Alerts::add_error(l('global.error_message.invalid_csrf_token'));
            redirect('vcards');
        }

        /* Verify the main resource */
        if(!$vcard = db()->where('vcard_id', $vcard_id)->where('user_id', $this->user->user_id)->getOne('vcards')) {
            redirect('vcards');
        }

        if(!Alerts::has_field_errors() && !Alerts::has_errors()) {

            /* Generate random url if not specified */
            $url = mb_strtolower(string_generate(settings()->vcards->random_url_length ?? 7));
            while(db()->where('url', $url)->getValue('vcards', 'vcard_id')) {
                $url = mb_strtolower(string_generate(settings()->vcards->random_url_length ?? 7));
            }

            /* Parse settings */
            $vcard->settings = json_decode($vcard->settings ?? '');

            /* Duplicate the files */
            $vcard->settings->logo = \Altum\Uploads::copy_uploaded_file($vcard->settings->logo, \Altum\Uploads::get_path('vcards/logo'), \Altum\Uploads::get_path('vcards/logo'));
            $vcard->settings->favicon = \Altum\Uploads::copy_uploaded_file($vcard->settings->favicon, \Altum\Uploads::get_path('vcards/favicon'), \Altum\Uploads::get_path('vcards/favicon'));
            $vcard->settings->opengraph = \Altum\Uploads::copy_uploaded_file($vcard->settings->opengraph, \Altum\Uploads::get_path('vcards/opengraph'), \Altum\Uploads::get_path('vcards/opengraph'));
            if($vcard->settings->background_type == 'image') $vcard->settings->background = \Altum\Uploads::copy_uploaded_file($vcard->settings->background, \Altum\Uploads::get_path('vcards/background'), \Altum\Uploads::get_path('vcards/background'));

            /* Insert to database */
            $vcard_id = db()->insert('vcards', [
                'user_id' => $this->user->user_id,
                'project_id' => $vcard->project_id,
                'pixels_ids' => $vcard->pixels_ids,
                'url' => $url,
                'name' => $vcard->name . ' - ' . l('global.duplicated'),
                'description' => $vcard->description,
                'settings' => json_encode($vcard->settings),
                'password' => $vcard->password,
                'theme' => $vcard->theme,
                'custom_js' => $vcard->custom_js,
                'custom_css' => $vcard->custom_css,
                'is_se_visible' => $vcard->is_se_visible,
                'is_removed_branding' => $vcard->is_removed_branding,
                'is_enabled' => $vcard->is_enabled,
                'datetime' => \Altum\Date::$date,
            ]);

            /* Get all blocks to duplicate */
            $vcard_blocks = db()->where('vcard_id', $vcard->vcard_id)->get('vcards_blocks');

            /* Duplicate all of them */
            foreach($vcard_blocks as $vcard_block) {
                /* Insert to database */
                $vcard_block_id = db()->insert('vcards_blocks', [
                    'vcard_id' => $vcard_id,
                    'user_id' => $this->user->user_id,
                    'type' => $vcard_block->type,
                    'name' => $vcard_block->name,
                    'value' => $vcard_block->value,
                    'settings' => $vcard_block->settings,
                    'order' => $vcard_block->order,
                    'is_enabled' => $vcard_block->is_enabled,
                    'datetime' => \Altum\Date::$date,
                ]);
            }

            /* Set a nice success message */
            Alerts::add_success(sprintf(l('global.success_message.create1'), '<strong>' . input_clean($vcard->name) . '</strong>'));

            /* Redirect */
            redirect('vcard-update/' . $vcard_id);

        }

        redirect('vcards');
    }

    public function bulk() {

        /* Check for any errors */
        if(empty($_POST)) {
            redirect('vcards');
        }

        if(empty($_POST['selected'])) {
            redirect('vcards');
        }

        if(!isset($_POST['type'])) {
            redirect('vcards');
        }

        //ALTUMCODE:DEMO if(DEMO) Alerts::add_error('This command is blocked on the demo.');

        if(!\Altum\Csrf::check()) {
            Alerts::add_error(l('global.error_message.invalid_csrf_token'));
        }

        if(!Alerts::has_field_errors() && !Alerts::has_errors()) {

            /* Team checks */
            if(\Altum\Teams::is_delegated() && !\Altum\Teams::has_access('delete.vcards')) {
                Alerts::add_info(l('global.info_message.team_no_access'));
                redirect('links');
            }

            foreach($_POST['selected'] as $vcard_id) {
                if($vcard = db()->where('vcard_id', $vcard_id)->where('user_id', $this->user->user_id)->getOne('vcards', ['vcard_id'])) {
                    (new \Altum\Models\Vcard())->delete($vcard_id);
                }
            }

            /* Set a nice success message */
            Alerts::add_success(l('bulk_delete_modal.success_message'));

        }

        redirect('vcards');
    }

    public function delete() {
        \Altum\Authentication::guard();

        /* Team checks */
        if(\Altum\Teams::is_delegated() && !\Altum\Teams::has_access('delete.vcards')) {
            Alerts::add_info(l('global.info_message.team_no_access'));
            redirect('vcards');
        }

        if(empty($_POST)) {
            redirect('vcards');
        }

        $vcard_id = (int) query_clean($_POST['vcard_id']);

        //ALTUMCODE:DEMO if(DEMO) if($this->user->user_id == 1) Alerts::add_error('Please create an account on the demo to test out this function.');

        if(!\Altum\Csrf::check()) {
            Alerts::add_error(l('global.error_message.invalid_csrf_token'));
            redirect('vcards');
        }

        /* Make sure the vcard id is created by the logged in user */
        if(!$vcard = db()->where('vcard_id', $vcard_id)->where('user_id', $this->user->user_id)->getOne('vcards', ['vcard_id', 'name'])) {
            redirect('vcards');
        }

        if(!Alerts::has_field_errors() && !Alerts::has_errors()) {

            (new \Altum\Models\Vcard())->delete($vcard->vcard_id);

            /* Set a nice success message */
            Alerts::add_success(sprintf(l('global.success_message.delete1'), '<strong>' . $vcard->name . '</strong>'));

            redirect('vcards');

        }

        redirect('vcards');
    }

}
